'use strict';

var crypto = require('crypto');

var _require = require('url'),
    parse = _require.parse,
    format = _require.format;

var encodeBase64Hash = function encodeBase64Hash(key, data) {
  return crypto.createHmac('sha1', key).update(data).digest('base64');
};

var expiracyIn = function expiracyIn(seconds) {
  return Math.floor(Date.now() / 1000) + seconds;
};

/**
 * Signature generating algorithm: (extracted from https://docs.aws.amazon.com/AmazonS3/latest/dev/RESTAuthentication.html#RESTAuthenticationQueryStringAuth)
 * 1. Create a string_to_sign containing the http verb, time to expire in
 *    seconds since epoch and route to object - for example:
 *    "GET\n\n\n1528921484\n/bucketname/object.png"
 *
 *    Note: use the same amount of line breaks, since they are headers. We
 *    are not taking them into consideration right now. Maybe later...
 * 2. Sign the string_to_sign using HMAC-SHA1 and the AWS secretAccessKey
 * 3. Encode the result into Base64
 * 4. Make resulting binary Url-safe
 * 5. Use this signature as the ?Signature= param in the url
 *
 * 6. You migh also need to add AWSAccessKeyId= and Expires= params to the url.
 *    At the end, the url will look like the following:
 *    https://s3.aws.com/bucket/obj.png?AWSAccessKeyId=XXXX&Signature=YYYY&Expires=12345
 *
 * @memberof module:Scripts
 * @param {string} s3_domain - For example https://s3-west.aws.com
 * @param {string} s3_bucket - S3 bucket
 * @param {string} s3_object_key - File name with extension
 * @param {string} AWSAccessKeyId - AWS Access Key ID
 * @param {string} AWSSecretAccessKey - AWS Access Key Secret
 * @param {number} secs_to_expire - Number of seconds in which signature will expire
 *                                  this param is set to 120secs by default
 * @returns {string}
 */
var signS3Object = function signS3Object(s3_domain, s3_bucket, s3_object_key, AWSAccessKeyId, AWSSecretAccessKey) {
  var secs_to_expire = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 120;

  var time_to_expire = expiracyIn(secs_to_expire);
  var string_to_sign = 'GET\n\n\n' + time_to_expire + '\n/' + s3_bucket + '/' + s3_object_key;

  var signature = encodeURIComponent(encodeBase64Hash(AWSSecretAccessKey, string_to_sign));

  return s3_domain + '/' + s3_bucket + '/' + s3_object_key + '?AWSAccessKeyId=' + AWSAccessKeyId + '&Signature=' + signature + '&Expires=' + time_to_expire;
};

module.exports = signS3Object;