'use strict';

/**
 * Formatter Utilities module.
 *
 * Common formatters that we use across different projects. Note that many methods
 * in this module use methods from other modules.
 * @module Formatters
 */
var get = require('lodash.get');

var _require = require('../general'),
    flow = _require.flow,
    toFixed = _require.toFixed,
    isNil = _require.isNil,
    isValidVarName = _require.isValidVarName;

var _require2 = require('../string'),
    addPostfix = _require2.addPostfix,
    separate = _require2.separate,
    capitalize = _require2.capitalize;

var sign = Math.sign,
    abs = Math.abs;
var isInteger = Number.isInteger;

/**
 * Separate input from right to left in buckets of 3 digits using specified delimiters.
 * Useful to format prices.
 *
 * @example
 * separateThousands(12000); //=>'12,000'
 * separateThousands('1250000'); //=> '1,250,000'
 * separateThousands(-1000); //=> '-1,000'
 * separateThousands(1000.111); //=> '1,000.111'
 *
 * Note that this function won't work with non numerical values. So expect:
 *
 * @example
 * separateThousands("hola"); //> 'hola'
 * separateThousands("0.123456"); //=> '0.123456'
 * separateThousands("1000.1234567"); //=> '1,000.1234567'
 * separateThousands("-100.10000"); //=> '-100.10000'
 *
 * @example
 * delimiters = { thousands: '.', decimal: ',' }
 * separateThousands(12000, delimiters); //=>'12.000'
 * separateThousands('1250000', delimiters); //=> '1.250.000'
 * separateThousands(-1000, delimiters); //=> '-1.000'
 * separateThousands(1000.111, delimiters); //=> '1.000,111'
 *
 * @param {number|string} value - Input that will be formatted with the specified delimiters
 * @param {object} delimiters - Object with delimiter settings. Default to { thousands: ',', decimal: '.' }
 * @returns {string}
 */

var separateThousands = function separateThousands(value) {
  var delimiters = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  if (isNil(value)) return null;
  if (isNaN(value)) return value;
  var thousandsSeparator = get(delimiters, 'thousands', ',');
  var decimalSeparator = get(delimiters, 'decimal', '.');

  var v_sign = sign(value) < 0 ? '-' : '';
  var integer = separate(parseInt(abs(value)), 3, thousandsSeparator, true);
  var floating_points = isInteger(Number(value)) ? '' : '' + decimalSeparator + ('' + value).split('.')[1];

  return '' + v_sign + integer + floating_points;
};

/**
* Run a pipeline of formatters to transform a string in snake_case style to camelCase
*
* @example
* snakeToCamelCase('snake_case_string'); //=> 'snakeCaseString'
* snakeToCamelCase('snake_case_string', true); //=> 'SnakeCaseString'
*
* @param {string} value - String in snake case style to be transformed to camel case
* @param {boolean} first_letter - Boolean that indicates if the first letter should be capitalized
* @returns {string}
*
*/
var snakeToCamelCase = function snakeToCamelCase(value) {
  var pascal_case = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  var pipeline = [function (word) {
    return word.split('_');
  }, function (words) {
    return words.map(function (w) {
      return capitalize(w);
    });
  }, function (words) {
    if (!pascal_case) {
      words[0] = words[0].toLowerCase();
    }return words;
  }, function (words) {
    return words.join('');
  }, function (varName) {
    if (!isValidVarName(varName)) {
      throw "Var name is not valid";
    } else return varName;
  }];
  return flow(pipeline)(value);
};

/**
 * Run a pipeline of formatters to transform a ranged 0 to 1 number into a
 * readable percentage.
 * The output would look like `XX.XX %`
 *
 * @example
 * toPercentage(0.67489); //=> '67.49 %'
 * toPercentage(0.67439, 1); //=> '67.4 %'
 *
 * @param {number} value - Floating precision number to transform into percentage
 * @param {number} decimal_points - Floating points to take in the final format
 * @returns {string}
 *
 */
var toPercentage = function toPercentage(value) {
  var decimal_points = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 2;
  return flow([function (x) {
    return x * 100;
  }, toFixed(decimal_points), addPostfix(" %")])(value);
};

//TODO: Add price, area, distance and range formatters

module.exports = {
  separateThousands: separateThousands,
  snakeToCamelCase: snakeToCamelCase,
  toPercentage: toPercentage
};